package com.ismasoft.controldiabetic.ui.activities

import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.PersistableBundle
import android.util.Log
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.get
import com.ismasoft.controldiabetic.R
import com.ismasoft.controldiabetic.databinding.ActivityMenuPrincipalBinding
import com.ismasoft.controldiabetic.ui.fragments.*
import com.ismasoft.controldiabetic.utilities.Constants.RETORN_ACTIVITY_OK_CODE
import com.ismasoft.controldiabetic.viewModel.MenuPrincipalViewModel

class MenuPrincipalActivity : AppCompatActivity() {

    private val mFragmentManager = supportFragmentManager

    private lateinit var viewModel: MenuPrincipalViewModel
    private lateinit var binding: ActivityMenuPrincipalBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_menu_principal)

        binding = ActivityMenuPrincipalBinding.inflate(layoutInflater)
        setContentView(binding.root)

        viewModel = ViewModelProvider(this).get()

        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        initNav()

    }

    override fun onSaveInstanceState(outState: Bundle, outPersistentState: PersistableBundle) {
        super.onSaveInstanceState(outState, outPersistentState)
        binding = ActivityMenuPrincipalBinding.inflate(layoutInflater)
        setContentView(binding.root)

        viewModel = ViewModelProvider(this).get()

        binding.viewModel = viewModel
        binding.lifecycleOwner = this

        initNav()
    }

    private fun initNav() {

        var activeFragment : Fragment

        var alarmesFragment = AlarmesFragment()
        var controlsFragment = ControlsFragment()
        var perfilGlobal = PerfilFragment()
        var visitesFragment = VisitesFragment()

        activeFragment = controlsFragment

        mFragmentManager.beginTransaction().add(R.id.frameLayout, alarmesFragment, "alarmes").hide(alarmesFragment).commit()
        mFragmentManager.beginTransaction().add(R.id.frameLayout, visitesFragment, "visites").hide(visitesFragment).commit()
        mFragmentManager.beginTransaction().add(R.id.frameLayout, perfilGlobal, "perfilPersonal").hide(perfilGlobal).commit()
        mFragmentManager.beginTransaction().add(R.id.frameLayout, controlsFragment, "controls").commit()

        this.binding.navigation.setOnNavigationItemSelectedListener {
            when (it.itemId) {
                R.id.controls -> {
                    mFragmentManager.beginTransaction()
                        .setCustomAnimations(R.anim.fade_scale_in, R.anim.fade_scale_out).hide(activeFragment).show(controlsFragment).commitNow()
                    activeFragment = controlsFragment
                    binding.fav.show()
                }
                R.id.alarmes -> {
                    mFragmentManager.beginTransaction()
                        .setCustomAnimations(R.anim.fade_scale_in, R.anim.fade_scale_out).hide(activeFragment).show(alarmesFragment).commitNow()
                    activeFragment = alarmesFragment
                    binding.fav.show()
                }
                R.id.visites-> {
                    mFragmentManager.beginTransaction()
                        .setCustomAnimations(R.anim.fade_scale_in, R.anim.fade_scale_out).hide(activeFragment).show(visitesFragment).commitNow()
                    activeFragment = visitesFragment
                    binding.fav.show()

                }
                R.id.perfil -> {
                    mFragmentManager.beginTransaction()
                        .setCustomAnimations(R.anim.fade_scale_in, R.anim.fade_scale_out).hide(activeFragment).show(perfilGlobal).commitNow()
                    activeFragment = perfilGlobal
                    binding.fav.hide()
                }
            }
            true
        }

        this.binding.navigation.setOnNavigationItemReselectedListener {
            Log.d("MainActivity", "Fragment reselected")
        }

        // Controls del botó afegir segons la pantalla del fragment
        binding.fav.setOnClickListener(){
            // Afegir control
            if(activeFragment.equals(controlsFragment)){
                intent = Intent(applicationContext, AfegirControlActivity::class.java)
                startActivityForResult(intent, RETORN_ACTIVITY_OK_CODE)
            }

            // Afegir alarma
            if(activeFragment.equals(alarmesFragment)){
                intent = Intent(applicationContext, AfegirAlarmaActivity::class.java)
                startActivityForResult(intent, RETORN_ACTIVITY_OK_CODE)
            }

            // Afegir visita
            if(activeFragment.equals(visitesFragment)){
                intent = Intent(applicationContext, AfegirVisitaActivity::class.java)
                startActivityForResult(intent, RETORN_ACTIVITY_OK_CODE)
            }
        }

    }

    override fun onBackPressed() {
        moveTaskToBack(true)
    }
}